@@ -389,7 +389,7 @@ sub parse_received_line {
 
   if (/^from /) {
     # try to catch enveloper senders
-    if (/envelope-(?:sender|from)[ =](\S+)\b/) {
+    if (/(?:return-path:? |envelope-(?:sender|from)[ =])(\S+)\b/i) {
       $envfrom = $1;
     }
 
@@ -401,6 +401,42 @@ sub parse_received_line {
       return;
     }
 
+    if (/\[XMail /) { # bug 3791
+      # Received: from list.brainbuzz.com (63.146.189.86:23198) by mx1.yourtech.net with [XMail 1.20 ESMTP Server] id <S72E> for <jason@ellingson.org> from <bounce-cscommunity-11965901@list.cramsession.com>; Sat, 18 Sep 2004 23:17:54 -0500
+      # Received: from list.brainbuzz.com (63.146.189.86:23198) by mx1.yourtech.net (209.32.147.34:25) with [XMail 1.20 ESMTP Server] id <S72E> for <jason@ellingson.org> from <bounce-cscommunity-11965901@list.cramsession.com>; Sat, 18 Sep 2004 23:17:54 -0500
+      if (/^from (\S+) \((${IP_ADDRESS})(?::\d+)?\) by (\S+)(?: \(\S+\)|) with \[XMail/)
+      {
+	$helo = $1; $ip = $2; $by = $3;
+        / id <(\S+)> / and $id = $1;
+        / from <(\S+)>; / and $envfrom = $1;
+        goto enough;
+      }
+    }
+
+    # catch MS-ish headers here
+    if (/ SMTPSVC/) {
+      # MS servers using this fmt do not lookup the rDNS.
+      # Received: from inet-vrs-05.redmond.corp.microsoft.com ([157.54.6.157])
+      # by INET-IMC-05.redmond.corp.microsoft.com with Microsoft
+      # SMTPSVC(5.0.2195.6624); Thu, 6 Mar 2003 12:02:35 -0800
+      # Received: from 0 ([61.31.135.91]) by bass.bass.com.eg with Microsoft
+      # SMTPSVC(5.0.2195.6713); Tue, 21 Sep 2004 08:59:06 +0300
+      # Received: from 0 ([61.31.138.57] RDNS failed) by nccdi.com with 
+      # Microsoft SMTPSVC(6.0.3790.0); Thu, 23 Sep 2004 08:51:06 -0700
+      # Received: from tthompson ([217.35.105.172] unverified) by
+      # mail.neosinteractive.com with Microsoft SMTPSVC(5.0.2195.5329);
+      # Tue, 11 Mar 2003 13:23:01 +0000
+      if (/^from (\S+) \(\[(${IP_ADDRESS})\][^\)]{0,40}\) by (\S+) with Microsoft SMTPSVC/) {
+        $helo = $1; $ip = $2; $by = $3; goto enough;
+      }
+
+      # Received: from mail pickup service by mail1.insuranceiq.com with
+      # Microsoft SMTPSVC; Thu, 13 Feb 2003 19:05:39 -0500
+      if (/^from mail pickup service by (\S+) with Microsoft SMTPSVC;/) {
+        return;
+      }
+    }
+
     if (/Exim/) {
       # one of the HUGE number of Exim formats :(
       # This must be scriptable.
@@ -449,7 +485,9 @@ sub parse_received_line {
     # Received: from 217.137.58.28 ([217.137.58.28])
     # by webmail.ukonline.net (IMP) with HTTP
     # for <anarchyintheuk@localhost>; Sun, 11 Apr 2004 00:31:07 +0100
-    if (/^from (${IP_ADDRESS}) \(\[${IP_ADDRESS}\]\) by (\S+).*\bwith HTTP/) {
+    if (/\bwith HTTP\b/ &&        # more efficient split up this way
+        /^from (${IP_ADDRESS}) \(\[${IP_ADDRESS}\]\) by (\S+)/)
+    {
       # some smarty-pants decided to fake a numeric HELO for HTTP
       # no rDNS for this format?
       $ip = $1; $by = $2; goto enough;
@@ -464,10 +502,8 @@ sub parse_received_line {
     }
 
     # from mail2.detr.gsi.gov.uk ([51.64.35.18] helo=ahvfw.dtlr.gsi.gov.uk) by mail4.gsi.gov.uk with smtp id 190K1R-0000me-00 for spamassassin-talk-admin@lists.sourceforge.net; Tue, 01 Apr 2003 12:33:46 +0100
-    if (/^from (\S+) \(\[(${IP_ADDRESS})\](.*)\) by (\S+) with /) {
-      $rdns = $1; $ip = $2; $by = $4;
-      my $sub = ' '.$3.' ';
-      if ($sub =~ / helo=(\S+) /) { $helo = $1; }
+    if (/^from (\S+) \(\[(${IP_ADDRESS})\] helo=(\S+)\) by (\S+) with /) {
+      $rdns = $1; $ip = $2; $helo = $3; $by = $4;
       goto enough;
     }
 
@@ -758,20 +794,6 @@ sub parse_received_line {
       $ip = $1; $by = $2; goto enough;
     }
 
-    # Received: from inet-vrs-05.redmond.corp.microsoft.com ([157.54.6.157]) by
-    # INET-IMC-05.redmond.corp.microsoft.com with Microsoft SMTPSVC(5.0.2195.6624);
-    # Thu, 6 Mar 2003 12:02:35 -0800
-    if (/^from (\S+) \(\[(${IP_ADDRESS})\]\) by (\S+) with /) {
-      $helo = $1; $ip = $2; $by = $3; goto enough;
-    }
-
-    # Received: from tthompson ([217.35.105.172] unverified) by
-    # mail.neosinteractive.com with Microsoft SMTPSVC(5.0.2195.5329);
-    # Tue, 11 Mar 2003 13:23:01 +0000
-    if (/^from (\S+) \(\[(${IP_ADDRESS})\] unverified\) by (\S+) with Microsoft SMTPSVC/) {
-      $helo = $1; $ip = $2; $by = $3; goto enough;
-    }
-
     # Received: from 157.54.8.23 by inet-vrs-05.redmond.corp.microsoft.com
     # (InterScan E-Mail VirusWall NT); Thu, 06 Mar 2003 12:02:35 -0800
     if (/^from (${IP_ADDRESS}) by (\S+) \(InterScan/) {
@@ -914,12 +936,6 @@ sub parse_received_line {
   # from qmail-scanner-general-admin@lists.sourceforge.net by alpha by uid 7791 with qmail-scanner-1.14 (spamassassin: 2.41. Clear:SA:0(-4.1/5.0):. Processed in 0.209512 secs)
   if (/^from \S+\@\S+ by \S+ by uid \S+ /) { return; }
 
-  # Received: from mail pickup service by mail1.insuranceiq.com with
-  # Microsoft SMTPSVC; Thu, 13 Feb 2003 19:05:39 -0500
-  if (/^from mail pickup service by (\S+) with Microsoft SMTPSVC;/) {
-    return;
-  }
-
   # Received: by x.x.org (bulk_mailer v1.13); Wed, 26 Mar 2003 20:44:41 -0600
   if (/^by (\S+) \(bulk_mailer /) { return; }
 
